from level import cell_width, cell_height, cell_hdepth, cell_vdepth
from level import Item, Gizmo, Wall
from level_io import register_class
from animation import Animation
from resource import get_sound

class DoorMixin(Item):
	#  frames       int      No. of frames in animation
	#  open_sound   Sound

	strips = 1
	open_sound = None

	def _get_params(self, p):
		p.frames = self.frames
		if self.open_sound_name:
			p.open_sound = self.open_sound_name

	def _set_params(self, p):
		self.frames = p.frames
		self.open_sound_name = getattr(p, 'open_sound', None)
		if self.open_sound_name:
			self.open_sound = get_sound(self.open_sound_name)
		self.anim = Animation(self.image_name, self.strips, self.frames, border = self.border)

	def begin_frame(self, level, coords):
		self.anim.begin_frame()
	
	def open(self):
		if not self.is_opening():
			self.anim.resume_forward()
			if self.open_sound:
				self.open_sound.play()
	
	def is_opening(self):
		return self.anim.running_forward()
	
	def close(self):
		self.anim.resume_backward()
	
	def set_open(self):
		self.anim.go_to_end()
	
	def set_closed(self):
		self.anim.go_to_beginning()
	
	def is_open(self):
		return self.anim.at_last_frame()


class BackDoor(Gizmo, DoorMixin):
	is_static = False
	editor_attrs = [("Tag", 'tag'), ("Up Link", 'link'), ("Down Link", 'down_link')]
	
	tag = ""
	link = ""
	down_link = ""

	def __init__(self, image, frames, border = 0, **kwds):
		Gizmo.__init__(self, image = image, frames = frames,
			border = border, **kwds)
	
	def _get_params(self, p):
		p.tag = self.tag
		p.link = self.link
		if self.down_link:
			p.down_link = self.down_link
	
	def _set_params(self, p):
		self.tag = getattr(p, 'tag', "")
		self.link = getattr(p, 'link', "")
		self.down_link = getattr(p, 'down_link', "")
	
	def draw(self, surface, rect, front = False):
		if not front:
			x, y = rect.bottomright
			pos = (x - cell_width // 2, y - cell_vdepth)
			self.anim.draw(surface, 'midbottom', pos)
	
	def get_link(self, down):
		if down:
			return self.down_link
		else:
			return self.link
	
	def player_has_entered(self):
		self.anim.start_from_end()


class SideDoor(Wall, DoorMixin):
	is_static = False
	strips = 2

	def __init__(self, image, frames, border = 0, **kwds):
		Wall.__init__(self, image = image, frames = frames,
			border = border, **kwds)
	
	def draw(self, surface, rect, front = False):
		if not front:
			self.anim.draw(surface, 'bottomright', rect.bottomright)
	
	def draw_behind(self, surface, rect):
		self.anim.draw(surface, 'bottomright', rect.bottomright, strip = 1)

	def passable(self):
		return self.is_open()
	
	def hit_by_player(self, player):
		self.open()
	

register_class(BackDoor)
register_class(SideDoor)
