from pygame import Rect, draw
from widget import Widget
from level import cell_size, cell_depth
from editor_constants import *

cw, ch = cell_size
dw, dh = cell_depth

class EditorLevelView(Widget):

	def __init__(self, editor, nrows, ncols):
		Widget.__init__(self, Rect(0, 0, ncols * cw + dw, nrows * ch + dh))
		self.editor = editor
	
	def draw(self, surface):
		level = self.editor.level
		level.draw(surface)
		pen = (128, 128, 128)
		x0 = 0
		x1 = cw * level.num_cols + dw
		for row in xrange(level.num_rows + 1):
			y = ch * row - 1
			draw.line(surface, pen, (x0, y), (x1, y))
			y += dh
			draw.line(surface, pen, (x0, y), (x1, y))
		y0 = 0
		y1 = ch * level.num_rows + dh
		for col in xrange(level.num_cols + 1):
			x = cw * col - 1
			draw.line(surface, pen, (x, y0), (x, y1))
			x += dw
			draw.line(surface, pen, (x, y0), (x, y1))
	
	def mouse_down(self, event):
		level = self.editor.level
		x, y = event.local
		col, dx = divmod(x, cw)
		row, dy = divmod(y, ch)
		if dx < dw:
			zone = ZONE_WALL
		elif dy < dh:
			zone = ZONE_FLOOR
		else:
			zone = ZONE_BACK
		if zone == ZONE_BACK:
			rlim, clim = level.num_rows, level.num_cols
		else:
			rlim, clim = level.num_rows + 1, level.num_cols + 1
		if 0 <= row < rlim and 0 <= col < clim:
			self.editor.click_cell(zone, row, col)
	
#	def change_back(self, pos, item):
#		coords = self.pos_to_back_coords(pos)
#		if coords:
#			self.editor.level.backs[coords] = item
#	
#	def pos_to_back_coords(self, (x, y)):
#		level = self.editor.level
#		row = (y - dh) // ch
#		col = (x - dw) // cw
#		if 0 <= row < level.num_rows and 0 <= col < level.num_cols:
#			return (row, col)
#		else:
#			return None
