import sys
from widget import Widget
from level_view import LevelView
from inventory_view import InventoryView
from charge_view import ChargeView
from player import Player
from level_loading import load_level
from resource import get_sound
from pygame.locals import *

window_size = (4, 13) # rows, cols
test_player_start = (3, 0)

alarm_sound = get_sound("alarm.wav")
time_to_capture = 60 # frames

class Game(Widget):
	#  levels     {name:Level}  loaded levels
	#  timeout    int           frames until captured, or 0

	def __init__(self, shell, start_level_name, start_cell = None):
		Widget.__init__(self, shell.rect)
		self.shell = shell
		self.player = Player()
		self.player.game = self
		level_view = LevelView(self, *window_size)
		inv_view = InventoryView(self.rect.width - 210, height = 100)
		inv_view.rect.topleft = level_view.rect.bottomleft
		y = level_view.rect.bottom + 5
		for kind, color in enumerate(["red", "green", "blue"]):
			charge_view = ChargeView(kind, color, 200, 20)
			charge_view.player = self.player
			charge_view.rect.topright = (self.rect.right - 5, y)
			y = charge_view.rect.bottom + 5
			self.add(charge_view)
		self.add(level_view)
		self.add(inv_view)
		inv_view.player = self.player
		self.levels = {}
		self.set_level(self.get_level(start_level_name))
		self.level.locate_player(start_cell or test_player_start)
		self.timeout = 0
		if self.shell.testing:
			make_test_inventory(self.player.inventory)
			self.player.cash = 10000

	def set_level(self, level):
		self.level = level
		level.set_player(self.player)

	def draw(self, surface):
		self.level.draw(surface)

	def key_down(self, event):
		key = event.key
		if key == K_ESCAPE:
			sys.exit(0)
		else:
			self.player.key_down(event)
	
	def key_up(self, event):
		self.player.key_up(event)
	
	def begin_frame(self):
		if self.timeout:
			self.timeout -= 1
			if self.timeout == 0:
				self.shell.player_captured()
		self.level.begin_frame()

	def switch_to_level(self, link):
		name, tag = parse_link(link, "entrance")
		if name <> self.level.filename:
			level = self.get_level(name)
			self.set_level(level)
		coords, door = self.level.find_tag(tag)
		self.level.locate_player(coords)
		if door:
			door.player_has_entered()
		else:
			print "*** Tag", link, "not found"
	
	def get_level(self, level_name):
		print "Getting level", level_name
		level = self.levels.get(level_name)
		if not level:
			print "Loading level", level_name
			level = load_level(level_name)
			self.levels[level_name] = level
		return level
	
	def sound_alarm(self):
		if not self.timeout:
			self.timeout = time_to_capture
			alarm_sound.play()


def parse_link(link, default_tag = ""):
	parts = link.split("#", 1)
	if len(parts) < 2:
		parts.append(default_tag)
	return parts

def make_test_inventory(inv):
	from level import Battery
	items = [
		Battery("red", "red_battery.png", "Red cloak battery"),
		Battery("green", "green_battery.png", "Green cloak battery"),
		Battery("blue", "blue_battery.png", "Blue cloak battery"),
	]
	for i in xrange(10):
		for c in items:
			inv.append(c.clone())
