import os, shutil
from level_io import level_path, Params

class OutputMapping(object):

	def __init__(self):
		self.items = []
		self.trans = {}
		self.next_index = 1
	
	def __getitem__(self, x):
		if x is None:
			return 0
		desc = "%s(%s)" % (x.__class__.__name__, x.get_params())
		if x.is_static:
			key = desc
		else:
			key = id(x)
		i = self.trans.get(key)
		if i is None:
			i = self.next_index
			self.next_index += 1
			self.items.append((i, desc))
			self.trans[key] = i
		return i


def write_array(f, label, a):
	mapping = OutputMapping()
	b = [[mapping[x] for x in row] for row in a]
	nitems = len(mapping.items)
	f.write("%s %d\n" % (label, nitems))
	for i, desc in mapping.items:
		f.write("%d: %s\n" % (i, desc))
	for row in b:
		for i in row:
			f.write(" %2d" % i)
		f.write("\n")

def write_level(f, lev):
	f.write("Level %d %d\n" % (lev.num_rows, lev.num_cols))
	write_array(f, "BackLayer", lev.backs)
	write_array(f, "WallLayer", lev.walls)
	write_array(f, "FloorLayer", lev.floors)
	write_array(f, "GizmoLayer", lev.gizmos)
	write_array(f, "ObjectLayer", lev.objects)
	f.write("EndLevel\n")

def save_level(lev, filename):
	path = level_path(filename)
	temp = os.tmpnam() + ".lvl"
	print "Using temp file", temp
	f = open(temp, "w")
	write_level(f, lev)
	f.close()
	shutil.copyfile(temp, path)
